package poly.plugins.mqo;
import java.io.*;
import poly.*;
import poly.plugins.*;

/**
 * MQO model data import plugin
 * 
 * @author Shun Moriya <shun126@users.sourceforge.jp>
 */
public class Mqo
{
	TokenReader tr;

	public void importFromFile(Scene scene, BufferedReader bufferedReader)
		throws FileNotFoundException, IOException
	{
		String token;
		
		tr = new TokenReader(bufferedReader);
		token = tr.getLine();
/*
		if("Metasequoia Document".equals(token) != true)
		{
			System.out.println("This file is not Metasequoia file.");
			return;
		}
*/
		token = tr.getLine();
		while((token = tr.getString()) != null)
		{
			if("Scene".equals(token))
			{
				ParseSceneChunk(scene);
			}else if("Material".equals(token)){
				parseMaterialChunk(scene);
			}else if("Object".equals(token)){
				ParseObjectChunk(scene);
			}
		}
	}

	void ParseSceneChunk(Scene scene) throws IOException
	{
		String token;

		while((token = tr.getString()) != null)
		{
			if("}".equals(token))
				break;
		}
	}

	void parseMaterialChunk(Scene scene) throws IOException
	{
		int iNumberOfMaterial = tr.getInt();

		tr.next();

		for(int i = 0; i < iNumberOfMaterial; i ++)
		{
			Material material = new Material();
			material.name = tr.getString();
			while(!tr.empty())
			{
				String token = tr.getString();
				if("col".equals(token))
				{
					material.r = tr.getFloat();
					material.g = tr.getFloat();
					material.b = tr.getFloat();
					material.a = tr.getFloat();
					
				}else if("dif".equals(token)){
					material.dif = tr.getFloat();
				}else if("amb".equals(token)){
					material.amb = tr.getFloat();
				}else if("emi".equals(token)){
					material.emi = tr.getFloat();
				}else if("spc".equals(token)){
					material.spc = tr.getFloat();
				}else if("power".equals(token)){
					material.power = tr.getFloat();
				}else if("tex".equals(token)){
					material.texture = tr.getString2();
				}
			}
			scene.appendMaterial(material);
		}
	}

	void ParseObjectChunk(Scene scene) throws IOException
	{
		Model model = new Model();
		String token;
		int visible = 0;
		
		token = tr.getString2();		// object name
		token = tr.getString();
		do{
			if("vertex".equals(token))
			{
				int iNumberOfVertex = tr.getInt();
				token = tr.getString();
				if("{".equals(token))
				{
					for(int i = 0; i < iNumberOfVertex; i ++)
					{
						model.appendVertex(tr.getFloat(), -tr.getFloat(), tr.getFloat());
					}
					token = tr.getString();
				}
			}else if ("visible".equals(token)){
				visible = tr.getInt();
			}else if ("color".equals(token)){
				tr.getFloat();
				tr.getFloat();
				tr.getFloat();
			}else if("face".equals(token)){
				int iNumberOfFace;
				
				iNumberOfFace = tr.getInt();
				token = tr.getString();
				if("{".equals(token))
				{
					for(int i = 0; i < iNumberOfFace; i ++)
					{
						Primitive primitive = new Primitive();
						int vertexCount = tr.getInt();
						switch(vertexCount)
						{
						case 3:
							primitive.quadrangle = false;
							break;
						case 4:
							primitive.quadrangle = true;
							break;
						}
						while(!tr.empty())
						{
							token = tr.getString();
							if("V".equals(token))
							{
								int iVertexId3;
								int iVertexId2;
								int iVertexId1;
								int iVertexId0;
								switch(vertexCount)
								{
								case 3:
									iVertexId2 = tr.getInt();
									iVertexId1 = tr.getInt();
									iVertexId0 = tr.getInt();
									primitive.vp0 = iVertexId2;
									primitive.vp1 = iVertexId1;
									primitive.vp2 = iVertexId0;
									break;
								case 4:
									iVertexId3 = tr.getInt();
									iVertexId2 = tr.getInt();
									iVertexId1 = tr.getInt();
									iVertexId0 = tr.getInt();
									primitive.vp0 = iVertexId3;
									primitive.vp1 = iVertexId2;
									primitive.vp2 = iVertexId1;
									primitive.vp3 = iVertexId0;
									break;
								}
							}else if("M".equals(token)){
								primitive.material = tr.getInt();
								primitive.enableTexture = true;
							}else if("UV".equals(token)){
								switch(vertexCount)
								{
								case 3:
									primitive.u0 = tr.getFloat();
									primitive.v0 = tr.getFloat();
									primitive.u1 = tr.getFloat();
									primitive.v1 = tr.getFloat();
									primitive.u2 = tr.getFloat();
									primitive.v2 = tr.getFloat();
									break;
								case 4:
									primitive.u0 = tr.getFloat();
									primitive.v0 = tr.getFloat();
									primitive.u1 = tr.getFloat();
									primitive.v1 = tr.getFloat();
									primitive.u2 = tr.getFloat();
									primitive.v2 = tr.getFloat();
									primitive.u3 = tr.getFloat();
									primitive.v3 = tr.getFloat();
									break;
								}
							}
						}
						model.appendPrimitive(primitive);
					}
				}
				token = tr.getString();
			}
			token = tr.getString();
		}while(!"}".equals(token));
		
		Bone bone;
		if(visible != 0)
		{
			bone = new Bone(model);
		}else{
			bone = new Bone();
		}
		scene.appendBone(bone);
	}
}
		
